@@ -7,7 +7,7 @@ NULL
 
 ## quiet concerns of R CMD check re: the .'s that appear in pipelines
 if (getRversion() >= "2.15.1") {
-  utils::globalVariables(c(".", "var", "maxl", "l", "seed"))
+  utils::globalVariables(c(".", "var", "maxl", "l", "seed", "frame"))
 }
 
 .onUnload <- function (libpath) {
