@@ -1,4 +1,4 @@
-#' Get the means/medians/variances of pillars of an
+#' Get the sums/means/medians/variances of pillars of an
 #' [ijtiff_img][ijtiff::ijtiff_img]-style array.
 #'
 #' For an [ijtiff_img][ijtiff::ijtiff_img]-style array `img` (indexed as `img[y,
@@ -14,6 +14,7 @@
 #'
 #' @examples
 #' aaa <- array(seq_len(2 ^ 4), dim = rep(2, 4))  # a 2-channel, 2-frame array
+#' sum_pillars(aaa)
 #' mean_pillars(aaa)
 #' median_pillars(aaa)
 #' var_pillars(aaa)
@@ -21,6 +22,27 @@
 #' @name pillar-stats
 NULL
 
+#' @rdname pillar-stats
+#' @export
+sum_pillars <- function(img, parallel = FALSE) {
+  checkmate::assert_numeric(img)
+  checkmate::assert_array(img, min.d = 3, max.d = 4)
+  d <- dim(img)
+  if (length(d) == 3) {
+    n_cores <- translate_parallel(parallel)
+    RcppParallel::setThreadOptions(n_cores)
+    on.exit(RcppParallel::setThreadOptions(RcppParallel::defaultNumThreads()))
+    out <- sum_pillars_(img)
+    d[3:4] <- 1
+  } else {
+    out <- purrr::map(seq_len(d[3]), ~ sum_pillars_(img[, , ., ])) %>%
+      unlist()
+    d[4] <- 1
+  }
+  dim(out) <- d
+  out
+}
+
 #' @rdname pillar-stats
 #' @export
 mean_pillars <- function(img, parallel = FALSE) {
@@ -31,14 +53,15 @@ mean_pillars <- function(img, parallel = FALSE) {
     n_cores <- translate_parallel(parallel)
     RcppParallel::setThreadOptions(n_cores)
     on.exit(RcppParallel::setThreadOptions(RcppParallel::defaultNumThreads()))
-    mean_pillars_(img)
+    out <- mean_pillars_(img)
+    d[3:4] <- 1
   } else {
     out <- purrr::map(seq_len(d[3]), ~ mean_pillars_(img[, , ., ])) %>%
       unlist()
     d[4] <- 1
-    dim(out) <- d
-    out
   }
+  dim(out) <- d
+  out
 }
 
 #' @rdname pillar-stats
@@ -51,14 +74,15 @@ median_pillars <- function(img, parallel = FALSE) {
     n_cores <- translate_parallel(parallel)
     RcppParallel::setThreadOptions(n_cores)
     on.exit(RcppParallel::setThreadOptions(RcppParallel::defaultNumThreads()))
-    median_pillars_(img)
+    out <- median_pillars_(img)
+    d[3:4] <- 1
   } else {
     out <- purrr::map(seq_len(d[3]), ~ median_pillars_(img[, , ., ])) %>%
       unlist()
     d[4] <- 1
-    dim(out) <- d
-    out
   }
+  dim(out) <- d
+  out
 }
 
 #' @rdname pillar-stats
@@ -71,14 +95,15 @@ var_pillars <- function(img, parallel = FALSE) {
     n_cores <- translate_parallel(parallel)
     RcppParallel::setThreadOptions(n_cores)
     on.exit(RcppParallel::setThreadOptions(RcppParallel::defaultNumThreads()))
-    var_pillars_(img)
+    out <- var_pillars_(img)
+    d[3:4] <- 1
   } else {
     out <- purrr::map(seq_len(d[3]), ~ var_pillars_(img[, , ., ])) %>%
       unlist()
     d[4] <- 1
-    dim(out) <- d
-    out
   }
+  dim(out) <- d
+  out
 }
 
 #' Get the brightness of pillars of a 3d array.
@@ -107,12 +132,24 @@ brightness_pillars <- function(img, parallel = FALSE) {
     n_cores <- translate_parallel(parallel)
     RcppParallel::setThreadOptions(n_cores)
     on.exit(RcppParallel::setThreadOptions(RcppParallel::defaultNumThreads()))
-    brightness_pillars_(img)
+    out <- brightness_pillars_(img)
+    d[3:4] <- 1
   } else {
     out <- purrr::map(seq_len(d[3]), ~ brightness_pillars_(img[, , ., ])) %>%
       unlist()
     d[4] <- 1
-    dim(out) <- d
-    out
+  }
+  dim(out) <- d
+  out
+}
+
+anyNA_pillars <- function(arr3d) {
+  checkmate::assert_array(arr3d, d = 3)
+  checkmate::assert(checkmate::check_integer(arr3d),
+                    checkmate::check_numeric(arr3d))
+  if (isTRUE(checkmate::check_integer(arr3d))) {
+    int_anyNA_pillars(arr3d)
+  } else {
+    dbl_anyNA_pillars(arr3d)
   }
 }
