@@ -1,4 +1,4 @@
-context("smooth")
+context("Smoothing")
 
 test_that("cpp smoothing functions work", {
   v <- runif(10)
@@ -9,5 +9,5 @@ test_that("cpp smoothing functions work", {
                weighted_smooth(v, exp5weights))
   v <- seq_len(3)
   expect_equal(weighted_smooth(v, rep(1, 21)), rep(2, 3))
-  expect_error(weighted_smooth(v, rep(1, 20)), "must be odd")
+  expect_error(weighted_smooth(v, rep(1, 20)), "must be odd|c\\+\\+ exception")
 })
